@@ -16,13 +16,14 @@ from pcs.lib.booth import (
 )
 from pcs.lib.booth.config_parser import parse, build
 from pcs.lib.booth.env import get_config_file_name
-from pcs.lib.cib.tools import get_resources
+from pcs.lib.cib.tools import get_resources, IdProvider
 from pcs.lib.communication.booth import (
     BoothGetConfig,
     BoothSendConfig,
 )
 from pcs.lib.communication.tools import run_and_raise
 from pcs.lib.errors import LibraryError, ReportItemSeverity
+from pcs.lib.node import get_existing_nodes_names
 from pcs.lib.resource_agent import find_valid_resource_agent_by_name
 
 
@@ -119,6 +120,9 @@ def config_text(env, node_name=None):
     com_cmd.set_targets([
         env.get_node_target_factory().get_target_from_hostname(node_name)
     ])
+    # pylint: disable=unsubscriptable-object
+    # In general, pylint is right. And it cannot know in this case code is OK.
+    # It is covered by tests.
     remote_data = run_and_raise(env.get_node_communicator(), com_cmd)[0][1]
     try:
         return remote_data["config"]["data"]
@@ -162,6 +166,7 @@ def create_in_cluster(env, ip, allow_absent_resource_agent=False):
         if its agent is not installed
     """
     resources_section = get_resources(env.get_cib())
+    id_provider = IdProvider(resources_section)
     name = env.booth.name
 
     booth_config_file_path = get_config_file_name(name)
@@ -183,6 +188,7 @@ def create_in_cluster(env, ip, allow_absent_resource_agent=False):
         primitive.create,
         env.report_processor,
         resources_section,
+        id_provider
     )
     into_booth_group = partial(
         group.place_resource,
@@ -264,6 +270,14 @@ def config_sync(env, skip_offline_nodes=False):
     authfile_content = config_files.read_authfile(
         env.report_processor, authfile_path
     )
+
+    cluster_nodes_names, report_list = get_existing_nodes_names(
+        env.get_corosync_conf()
+    )
+    if not cluster_nodes_names:
+        report_list.append(reports.corosync_config_no_nodes_defined())
+    env.report_processor.process_list(report_list)
+
     com_cmd = BoothSendConfig(
         env.report_processor,
         env.booth.name,
@@ -274,7 +288,7 @@ def config_sync(env, skip_offline_nodes=False):
     )
     com_cmd.set_targets(
         env.get_node_target_factory().get_target_list(
-            env.get_corosync_conf().get_nodes_names(),
+            cluster_nodes_names,
             skip_non_existing=skip_offline_nodes,
         )
     )
@@ -378,6 +392,9 @@ def pull_config(env, node_name):
     com_cmd.set_targets([
         env.get_node_target_factory().get_target_from_hostname(node_name)
     ])
+    # pylint: disable=unsubscriptable-object
+    # In general, pylint is right. And it cannot know in this case code is OK.
+    # It is covered by tests.
     output = run_and_raise(env.get_node_communicator(), com_cmd)[0][1]
     try:
         env.booth.create_config(output["config"]["data"], True)
