@@ -1,23 +1,16 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
+import re
 from collections import namedtuple
 from lxml import etree
-import os
-import re
 
 from pcs import settings
 from pcs.common import report_codes
 from pcs.common.tools import xml_fromstring
-from pcs.lib import reports
+from pcs.lib import reports, validate
 from pcs.lib.errors import LibraryError, ReportItemSeverity
 from pcs.lib.pacemaker.values import is_true
 
-
-_crm_resource = os.path.join(settings.pacemaker_binaries, "crm_resource")
+# TODO: fix
+# pylint: disable=no-self-use
 
 # Operation monitor is required always! No matter if --no-default-ops was
 # entered or if agent does not specify it. See
@@ -137,7 +130,7 @@ def list_resource_agents_standards(runner):
     """
     # retval is number of standards found
     stdout, dummy_stderr, dummy_retval = runner.run([
-        _crm_resource, "--list-standards"
+        settings.crm_resource_binary, "--list-standards"
     ])
     ignored_standards = frozenset([
         # we are only interested in RESOURCE agents
@@ -153,7 +146,7 @@ def list_resource_agents_ocf_providers(runner):
     """
     # retval is number of providers found
     stdout, dummy_stderr, dummy_retval = runner.run([
-        _crm_resource, "--list-ocf-providers"
+        settings.crm_resource_binary, "--list-ocf-providers"
     ])
     return _prepare_agent_list(stdout)
 
@@ -191,7 +184,7 @@ def list_resource_agents(runner, standard_provider):
     """
     # retval is 0 on success, anything else when no agents found
     stdout, dummy_stderr, retval = runner.run([
-        _crm_resource, "--list-agents", standard_provider
+        settings.crm_resource_binary, "--list-agents", standard_provider
     ])
     if retval != 0:
         return []
@@ -205,7 +198,7 @@ def list_stonith_agents(runner):
     """
     # retval is 0 on success, anything else when no agents found
     stdout, dummy_stderr, retval = runner.run([
-        _crm_resource, "--list-agents", "stonith"
+        settings.crm_resource_binary, "--list-agents", "stonith"
     ])
     if retval != 0:
         return []
@@ -328,6 +321,7 @@ def _find_valid_agent_by_name(
     report_processor, runner, name, PresentAgentClass, AbsentAgentClass,
     absent_agent_supported=True
 ):
+    # pylint: disable=invalid-name
     try:
         return PresentAgentClass(runner, name).validate_metadata()
     except (InvalidResourceAgentName, InvalidStonithAgentName) as e:
@@ -349,10 +343,12 @@ def _find_valid_agent_by_name(
 
         return AbsentAgentClass(runner, name)
 
-class Agent(object):
+class Agent():
     """
     Base class for providing convinient access to an agent's metadata
     """
+    _agent_type_label = "agent"
+
     def __init__(self, runner):
         """
         create an instance which reads metadata by itself on demand
@@ -426,22 +422,42 @@ class Agent(object):
         """
         Get list of agent's parameters, each parameter is described by dict:
         {
-            name: name of parameter
+            name: name of the parameter
             longdesc: long description,
             shortdesc: short description,
-            type: data type od parameter,
+            type: data type of the parameter,
             default: default value,
-            required: True if is required parameter, False otherwise
+            required: True if it is a required parameter, False otherwise
+            advanced: True if the parameter is considered for advanced users
+            deprecated: True if it is a deprecated parameter, False otherwise
+            deprecated_by: list of parameters deprecating this one
+            obsoletes: name of a deprecated parameter obsoleted by this one
+            unique: True if the parameter's value should be unique across same
+                agent resources, False otherwise
+            pcs_deprecated_warning: pcs originated warning
         }
         """
         params_element = self._get_metadata().find("parameters")
         if params_element is None:
             return []
+
         param_list = []
+        deprecated_by_dict = {}
         for param_el in params_element.iter("parameter"):
             param = self._get_parameter(param_el)
-            if not param["obsoletes"]:
-                param_list.append(param)
+            param_list.append(param)
+            if param["obsoletes"]:
+                obsoletes = param["obsoletes"]
+                if not obsoletes in deprecated_by_dict:
+                    deprecated_by_dict[obsoletes] = set()
+                deprecated_by_dict[obsoletes].add(param["name"])
+
+        for param in param_list:
+            if param["name"] in deprecated_by_dict:
+                param["deprecated_by"] = sorted(
+                    deprecated_by_dict[param["name"]]
+                )
+
         return param_list
 
 
@@ -464,70 +480,184 @@ class Agent(object):
             "type": value_type,
             "default": default_value,
             "required": is_true(parameter_element.get("required", "0")),
-            "advanced": False,
             "deprecated": is_true(parameter_element.get("deprecated", "0")),
             "obsoletes": parameter_element.get("obsoletes", None),
+            "unique": is_true(parameter_element.get("unique", "0")),
         })
 
-    def validate_parameters(
-        self, parameters,
-        parameters_type="resource",
-        allow_invalid=False,
-        update=False
-    ):
-        forceable = report_codes.FORCE_OPTIONS if not allow_invalid else None
-        severity = (
-            ReportItemSeverity.ERROR if not allow_invalid
-            else ReportItemSeverity.WARNING
+    def _get_parameter_obsoleting_chains(self):
+        """
+        get a dict describing parameters obsoleting
+
+        Each key is a parameter which obsoletes parameters but is not itself
+        obsoleted by any other parameter. Values are lists of obsoleted
+        parameters: the first one is obsoleted by the key, the second one is
+        obsoleted by the first one and so on.
+        """
+        # In meta-data, each param can have 'obsoletes' attribute containing a
+        # name of a single param obsoleted by the param in question. That means:
+        # 1) a deprecated param can be obsoleted by more than one param
+        # 2) a param can obsolete none or one param
+        # 3) there may be a loop (A obsoletes B, B obsoletes A)
+        # This info is crucial when dealing with obsoleting and deprecated
+        # params and it is reflected in the following piece of code.
+
+        # first, get simple obsoleting mapping
+        new_old = {
+            param["name"]: param["obsoletes"]
+            for param in self.get_parameters()
+            if param["obsoletes"]
+        }
+
+        chains = {}
+        # for each param which is not obsoleted
+        for new_param in new_old:
+            # start a new chain
+            deprecated_by_new = []
+            current_new = new_param
+            # while current parameter obsoletes anything
+            while current_new in new_old:
+                # get the obsoleted parameter
+                old = new_old[current_new]
+                # if there is a loop, break
+                if old in deprecated_by_new or old == current_new:
+                    break
+                # add the obsoleted parameter to the chain
+                deprecated_by_new.append(old)
+                # check if the obsoleted parameter obsoletes anything
+                current_new = old
+            if deprecated_by_new:
+                chains[new_param] = deprecated_by_new
+        return chains
+
+    def validate_parameters_create(self, parameters, force=False):
+        # This is just a basic validation checking that required parameters are
+        # set and all set parameters are known to an agent. Missing checks are:
+        # 1. values checks - if a param is an integer, then "abc" is not valid
+        # 2. warnings should be emitted when a deprecated param is set
+        # 3. errors should be emitted when a deprecated parameter and a
+        #    parameter obsoleting it are set at the same time
+        # 4. possibly some other checks
+        # All of these have been missing in pcs since ever (ad 1. agents have
+        # never provided enough info for us to do such validations, ad 2. and
+        # 3. there were no deprecated parameters before). The checks should be
+        # implemented in agents themselves, so I'm not adding them now either.
+        report_items = []
+
+        # report unknown parameters
+        report_items.extend(
+            validate.NamesIn(
+                {param["name"] for param in self.get_parameters()},
+                option_type=self._agent_type_label,
+                **validate.set_warning(report_codes.FORCE_OPTIONS, force)
+            ).validate(parameters)
         )
 
-        report_list = []
-        bad_opts, missing_req_opts = self.validate_parameters_values(
+        # report missing required parameters
+        missing_parameters = self._find_missing_required_parameters(
             parameters
         )
-
-        if bad_opts:
-            report_list.append(reports.invalid_options(
-                bad_opts,
-                sorted([attr["name"] for attr in self.get_parameters()]),
-                parameters_type,
+        if missing_parameters:
+            forcible, severity = self._validate_report_forcible_severity(force)
+            report_items.append(reports.required_options_are_missing(
+                sorted(missing_parameters),
+                self._agent_type_label,
                 severity=severity,
-                forceable=forceable,
+                forceable=forcible,
             ))
 
-        if not update and missing_req_opts:
-            report_list.append(reports.required_option_is_missing(
-                missing_req_opts,
-                parameters_type,
-                severity=severity,
-                forceable=forceable,
-            ))
+        return report_items
 
-        return report_list
+    def validate_parameters_update(
+        self,
+        current_parameters,
+        new_parameters,
+        force=False
+    ):
+        # This is just a basic validation checking that required parameters are
+        # set and all set parameters are known to an agent. Missing checks are:
+        # 1. values checks - if a param is an integer, then "abc" is not valid
+        # 2. warnings should be emitted when a deprecated param is set
+        # 3. errors should be emitted when a deprecated parameter and a
+        #    parameter obsoleting it are set at the same time
+        # 4. possibly some other checks
+        # All of these have been missing in pcs since ever (ad 1. agents have
+        # never provided enough info for us to do such validations, ad 2. and
+        # 3. there were no deprecated parameters before). The checks should be
+        # implemented in agents themselves, so I'm not adding them now either.
+        report_items = []
 
-    def validate_parameters_values(self, parameters_values):
-        """
-        Return tuple of lists (<invalid attributes>, <missing required attributes>)
-        dict parameters_values key is attribute name and value is attribute value
-        """
-        # TODO Add value and type checking (e.g. if parameter["type"] is
-        # integer, its value cannot be "abc"). This most probably will require
-        # redefining the format of the return value and rewriting the whole
-        # function, which will only be good. For now we just stick to the
-        # original legacy code.
-        agent_params = self.get_parameters()
+        # get resulting set of agent's parameters
+        final_parameters = dict(current_parameters)
+        for name, value in new_parameters.items():
+            if value:
+                final_parameters[name] = value
+            else:
+                if name in final_parameters:
+                    del final_parameters[name]
 
-        required_missing = []
-        for attr in agent_params:
-            if attr["required"] and attr["name"] not in parameters_values:
-                required_missing.append(attr["name"])
-
-        valid_attrs = [attr["name"] for attr in agent_params]
-        return (
-            [attr for attr in parameters_values if attr not in valid_attrs],
-            required_missing
+        # report unknown parameters
+        report_items.extend(
+            validate.NamesIn(
+                {param["name"] for param in self.get_parameters()},
+                option_type=self._agent_type_label,
+                **validate.set_warning(report_codes.FORCE_OPTIONS, force)
+            ).validate(
+                # Do not report unknown parameters already set in the CIB. They
+                # have been reported already when the were added to the CIB.
+                {
+                    name: value for name, value in new_parameters.items()
+                    if name not in current_parameters
+                }
+            )
         )
 
+        # report missing or removed required parameters
+        missing_parameters = self._find_missing_required_parameters(
+            final_parameters
+        )
+        if missing_parameters:
+            forcible, severity = self._validate_report_forcible_severity(force)
+            report_items.append(reports.required_options_are_missing(
+                sorted(missing_parameters),
+                self._agent_type_label,
+                severity=severity,
+                forceable=forcible,
+            ))
+
+        return report_items
+
+    def _validate_report_forcible_severity(self, force):
+        forcible = report_codes.FORCE_OPTIONS if not force else None
+        severity = (
+            ReportItemSeverity.ERROR if not force
+            else ReportItemSeverity.WARNING
+        )
+        return forcible, severity
+
+    def _find_missing_required_parameters(self, parameters):
+        missing_parameters = set()
+        obsoleting_chains = self._get_parameter_obsoleting_chains()
+        for param in self.get_parameters():
+            if not param["required"] or param["deprecated_by"]:
+                # non-required params are never required
+                # we require non-deprecated params preferentially
+                continue
+            if param["name"] in parameters:
+                # the param is not missing
+                continue
+            # the param is missing, maybe a deprecated one is set instead?
+            if param["name"] in obsoleting_chains:
+                obsoleted_set_instead = False
+                for obsoleted_name in obsoleting_chains[param["name"]]:
+                    if obsoleted_name in parameters:
+                        obsoleted_set_instead = True
+                        break
+                if obsoleted_set_instead:
+                    continue
+            missing_parameters.add(param["name"])
+        return missing_parameters
+
     def _get_raw_actions(self):
         actions_element = self._get_metadata().find("actions")
         if actions_element is None:
@@ -557,6 +687,7 @@ class Agent(object):
         return action_list
 
     def _is_cib_default_action(self, action):
+        # pylint: disable=unused-argument
         return False
 
     def get_cib_default_actions(self, necessary_only=False):
@@ -630,8 +761,10 @@ class Agent(object):
             "required": False,
             "advanced": False,
             "deprecated": False,
+            "deprecated_by": [],
             "obsoletes": None,
             "pcs_deprecated_warning": "",
+            "unique": False,
         }
         new_param.update(properties)
         return new_param
@@ -642,13 +775,13 @@ class FakeAgentMetadata(Agent):
     pass
 
 
-class StonithdMetadata(FakeAgentMetadata):
+class FencedMetadata(FakeAgentMetadata):
     def get_name(self):
-        return "stonithd"
+        return "pacemaker-fenced"
 
 
     def _get_parameter(self, parameter_element):
-        parameter = super(StonithdMetadata, self)._get_parameter(
+        parameter = super(FencedMetadata, self)._get_parameter(
             parameter_element
         )
         # Metadata are written in such a way that a longdesc text is a
@@ -665,7 +798,7 @@ class StonithdMetadata(FakeAgentMetadata):
 
     def _load_metadata(self):
         stdout, stderr, dummy_retval = self._runner.run(
-            [settings.stonithd_binary, "metadata"]
+            [settings.pacemaker_fenced, "metadata"]
         )
         metadata = stdout.strip()
         if not metadata:
@@ -727,7 +860,11 @@ class CrmAgent(Agent):
             "/usr/bin/",
         ])
         stdout, stderr, retval = self._runner.run(
-            [_crm_resource, "--show-metadata", self._get_full_name()],
+            [
+                settings.crm_resource_binary,
+                "--show-metadata",
+                self._get_full_name(),
+            ],
             env_extend={
                 "PATH": env_path,
             }
@@ -741,6 +878,8 @@ class ResourceAgent(CrmAgent):
     """
     Provides convinient access to a resource agent's metadata
     """
+    _agent_type_label = "resource"
+
     def _prepare_name_parts(self, name):
         return get_resource_agent_name_from_string(name)
 
@@ -818,12 +957,22 @@ class ResourceAgent(CrmAgent):
         ]
 
 
-class AbsentAgentMixin(object):
+class AbsentAgentMixin():
     def _load_metadata(self):
         return "<resource-agent/>"
 
-    def validate_parameters_values(self, parameters_values):
-        return ([], [])
+    def validate_parameters_create(self, parameters, force=False):
+        # pylint: disable=unused-argument
+        return []
+
+    def validate_parameters_update(
+        # pylint: disable=unused-argument
+        self,
+        current_parameters,
+        new_parameters,
+        force=False
+    ):
+        return []
 
 
 class AbsentResourceAgent(AbsentAgentMixin, ResourceAgent):
@@ -834,11 +983,12 @@ class StonithAgent(CrmAgent):
     """
     Provides convinient access to a stonith agent's metadata
     """
-    _stonithd_metadata = None
+    _fenced_metadata = None
+    _agent_type_label = "stonith"
 
     @classmethod
-    def clear_stonithd_metadata_cache(cls):
-        cls._stonithd_metadata = None
+    def clear_fenced_metadata_cache(cls):
+        cls._fenced_metadata = None
 
     def _prepare_name_parts(self, name):
         # pacemaker doesn't support stonith (nor resource) agents with : in type
@@ -855,35 +1005,46 @@ class StonithAgent(CrmAgent):
                 super(StonithAgent, self).get_parameters()
             )
             +
-            self._get_stonithd_metadata().get_parameters()
+            self._get_fenced_metadata().get_parameters()
         )
 
-    def validate_parameters(
-        self, parameters,
-        parameters_type="stonith",
-        allow_invalid=False,
-        update=False
-    ):
-        report_list = super(StonithAgent, self).validate_parameters(
+    def validate_parameters_create(self, parameters, force=False):
+        report_list = super(StonithAgent, self).validate_parameters_create(
             parameters,
-            parameters_type=parameters_type,
-            allow_invalid=allow_invalid,
-            update=update
+            force=force
         )
+        report_list.extend(
+            self._validate_action_is_deprecated(parameters, force=force)
+        )
+        return report_list
+
+    def validate_parameters_update(
+        self,
+        current_parameters,
+        new_parameters,
+        force=False
+    ):
+        report_list = super(StonithAgent, self).validate_parameters_update(
+            current_parameters,
+            new_parameters,
+            force=force
+        )
+        report_list.extend(
+            self._validate_action_is_deprecated(new_parameters, force=force)
+        )
+        return report_list
+
+    def _validate_action_is_deprecated(self, parameters, force=False):
         if parameters.get("action", ""):
-            report_list.append(reports.deprecated_option(
+            forcible, severity = self._validate_report_forcible_severity(force)
+            return [reports.deprecated_option(
                 "action",
                 _STONITH_ACTION_REPLACED_BY,
-                parameters_type,
-                severity=(
-                    ReportItemSeverity.ERROR if not allow_invalid
-                    else ReportItemSeverity.WARNING
-                ),
-                forceable=(
-                    report_codes.FORCE_OPTIONS if not allow_invalid else None
-                )
-            ))
-        return report_list
+                self._agent_type_label,
+                severity=severity,
+                forceable=forcible
+            )]
+        return []
 
     def _filter_parameters(self, parameters):
         """
@@ -918,16 +1079,34 @@ class StonithAgent(CrmAgent):
                 filtered.append(new_param)
             else:
                 filtered.append(param)
-            # 'port' parameter is required by a fence agent, but it is filled
-            # automatically by pacemaker based on 'pcmk_host_map' or
-            # 'pcmk_host_list' parameter (defined in stonithd metadata).
-            # Pacemaker marks the 'port' parameter as not required for us.
+
+        # 'port' parameter is required by a fence agent, but it is filled
+        # automatically by pacemaker based on 'pcmk_host_map' or
+        # 'pcmk_host_list' parameter (defined in fenced metadata). Pacemaker
+        # marks 'port' parameter as not required for us.
+        # It, however, doesn't mark any parameters deprecating 'port' as not
+        # required se we must do so ourselves.
+        port_related_params = set(["port"])
+        new_deprecated = set(["port"])
+        while new_deprecated:
+            current_deprecated = new_deprecated
+            new_deprecated = set()
+            for param in filtered:
+                if param["obsoletes"] in current_deprecated:
+                    if param["name"] not in port_related_params:
+                        new_deprecated.add(param["name"])
+                        port_related_params.add(param["name"])
+        for param in filtered:
+            if param["name"] in port_related_params:
+                param["required"] = False
+
         return filtered
 
-    def _get_stonithd_metadata(self):
-        if not self.__class__._stonithd_metadata:
-            self.__class__._stonithd_metadata = StonithdMetadata(self._runner)
-        return self.__class__._stonithd_metadata
+    def _get_fenced_metadata(self):
+        # pylint: disable=protected-access
+        if not self.__class__._fenced_metadata:
+            self.__class__._fenced_metadata = FencedMetadata(self._runner)
+        return self.__class__._fenced_metadata
 
     def get_provides_unfencing(self):
         # self.get_actions returns an empty list
