@@ -7,7 +7,6 @@ pacemaker_binaries = "/usr/sbin/"
 crm_resource_binary = os.path.join(pacemaker_binaries, "crm_resource")
 corosync_binaries = "/usr/sbin/"
 corosync_qnet_binaries = "/usr/bin/"
-ccs_binaries = "/usr/sbin/"
 corosync_conf_dir = "/etc/corosync/"
 corosync_conf_file = os.path.join(corosync_conf_dir, "corosync.conf")
 corosync_uidgid_dir = os.path.join(corosync_conf_dir, "uidgid.d/")
@@ -33,7 +32,7 @@ pacemaker_schedulerd = "/usr/libexec/pacemaker/pacemaker-schedulerd"
 pacemaker_controld = "/usr/libexec/pacemaker/pacemaker-controld"
 pacemaker_based = "/usr/libexec/pacemaker/pacemaker-based"
 pacemaker_fenced = "/usr/libexec/pacemaker/pacemaker-fenced"
-pcs_version = "0.10.1"
+pcs_version = "0.10.2"
 crm_report = os.path.join(pacemaker_binaries, "crm_report")
 crm_verify = os.path.join(pacemaker_binaries, "crm_verify")
 cibadmin = os.path.join(pacemaker_binaries, "cibadmin")
@@ -46,6 +45,7 @@ pcsd_settings_conf_location = "/var/lib/pcsd/pcs_settings.conf"
 pcsd_exec_location = "/usr/lib/pcsd/"
 pcsd_log_location = "/var/log/pcsd/pcsd.log"
 pcsd_default_port = 2224
+pcsd_config = "/etc/sysconfig/pcsd"
 cib_dir = "/var/lib/pacemaker/cib/"
 pacemaker_uname = "hacluster"
 pacemaker_gname = "haclient"
