@@ -1,12 +1,6 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
 from collections import namedtuple
-
-import pcs.snmp.agentx.pcs_pyagentx as pyagentx
+# pylint: disable=import-error
+import pyagentx
 
 
 BaseType = namedtuple("BaseType", ["data_type", "value"])
