@@ -1,4 +1,3 @@
-require 'digest/sha2'
 require 'logger'
 require 'open4'
 require 'pathname'
@@ -6,23 +5,6 @@ require 'pathname'
 require 'settings.rb'
 
 
-def is_rhel6()
-  # Checking corosync version works in most cases and supports non-rhel
-  # distributions as well as running (manually compiled) corosync2 on rhel6.
-  # - corosync2 does not support cman at all
-  # - corosync1 runs with cman on rhel6
-  # - corosync1 can be used without cman, but we don't support it anyways
-  # - corosync2 is the default result if errors occur
-  out = ''
-  status = Open4::popen4(COROSYNC, '-v') { |pid, stdin, stdout, stderr|
-    out = stdout.readlines()
-  }
-  retval = status.exitstatus
-  return false if retval != 0
-  match = /version\D+(\d+)/.match(out.join())
-  return (match and match[1] == "1")
-end
-
 def is_systemctl()
   systemctl_paths = [
       '/run/systemd/system',
@@ -49,17 +31,20 @@ def get_pcs_path()
   end
 end
 
-PCS_VERSION = '0.9.164'
-# unique instance signature, allows detection of dameon restarts
-DAEMON_INSTANCE_SIGNATURE = Digest::SHA2.hexdigest("#{Time.now} #{rand()}")
-COROSYNC = COROSYNC_BINARIES + "corosync"
-ISRHEL6 = is_rhel6
-ISSYSTEMCTL = is_systemctl
-if ISRHEL6
-  COROSYNC_CMAPCTL = COROSYNC_BINARIES + "corosync-objctl"
-else
-  COROSYNC_CMAPCTL = COROSYNC_BINARIES + "corosync-cmapctl"
+def get_pcs_internal_path()
+  pcsd_path = get_pcsd_path().to_s
+  if PCSD_EXEC_LOCATION == pcsd_path or PCSD_EXEC_LOCATION == (pcsd_path + '/')
+    return PCS_INTERNAL_EXEC
+  else
+    return pcsd_path + '/../pcs/pcs_internal'
+  end
 end
+
+PCS_VERSION = '0.10.2'
+# unique instance signature, allows detection of dameon restarts
+COROSYNC = COROSYNC_BINARIES + "corosync"
+ISSYSTEMCTL = is_systemctl
+COROSYNC_CMAPCTL = COROSYNC_BINARIES + "corosync-cmapctl"
 COROSYNC_QUORUMTOOL = COROSYNC_BINARIES + "corosync-quorumtool"
 
 if not defined? $cur_node_name
@@ -75,12 +60,6 @@ def configure_logger(log_device)
     logger.level = Logger::INFO
   end
 
-  if ISRHEL6
-    logger.debug "Detected RHEL 6"
-  else
-    logger.debug "Did not detect RHEL 6"
-  end
-
   if ISSYSTEMCTL
     logger.debug "Detected systemd is in use"
   else
